from nonebot import get_plugin_config, get_driver
from pydantic import BaseModel
from typing import List


class Config(BaseModel):
    # 配置项信息
    zepplife_user: str
    zepplife_password: str
    superusers: List[str]
    # 相关指令
    zepplife_commands: dict[str, str | list[str]] = {
        "help": ["刷步帮助", "stephelp"],  # 刷步帮助
        "check": ["刷步检查", "stepcheck"],  # 刷步检查
        "auto_step": ["自动刷步", "autostep"],  # 自动刷步
        "manual_step": ["手动刷步", "manualstep"],  # 手动刷步
    }
    # 接口地址
    url: str = "https://free.xwteam.cn/api/wechat/step"
    # 权限设置
    private_chat: bool = True  # 允许私聊
    group_chat: bool = True  # 允许群聊
    group_at: bool = True  # 群聊回复是否at_sender
    only_superusers_used: bool = False  # 仅超级用户可使用
    # 其他设置
    handle_module: bool = True  # 是否输出详情，推荐调试时使用
    message_block_requesterror: str = "服务器请求失败，请稍后再试。"
    message_block_unknownerror: str = "发生了未知错误！（肯定是服务器的问题喵~）请检查是否刷步成功，若成功则忽略该条信息。"
    message_success: str = "步数修改成功！\n\nTips:建议刷步时间每次间隔30分钟，防止封号。"
    message_block_step: str = "步数输入无效，请重新输入一个不超过98800的纯数字组成的数。"
    message_block_chinesecomma: str = "请重新输入，不要使用中文逗号！"
    message_loading: str = "正在修改中..."
    message_help: str = "刷步方式：向机器人发送对应的中文或英文指令后按提示操作即可。\n\nstepcheck: 刷步检查\n\nmanualstep: 手动刷步\n\nautostep: 自动刷步"
    message_block_users: str = "权限不足，请联系管理员处理。"
    message_block_private: str = "私聊功能已关闭。如有需要，请联系管理员处理。"
    message_block_config: str = "缺少必要的配置项，请检查配置文件中的关键字是否正确填写。"
    message_block_group: str = "群聊功能已关闭。如有需要，请联系管理员处理。"


conf = get_plugin_config(Config)
config = get_driver().config



import httpx
from httpx import AsyncClient
from nonebot.adapters.onebot.v11 import Message
from nonebot.matcher import Matcher
from nonebot.log import logger
from .Config import conf
from .ResultModule import load_module

handle_module = conf.handle_module
user = conf.zepplife_user
password = conf.zepplife_password
url = conf.url
message_success = conf.message_success
message_block_requesterror = conf.message_block_requesterror
message_block_unknownerror = conf.message_block_unknownerror
message_block_step = conf.message_block_step
message_block_chinesecomma = conf.message_block_chinesecomma
message_loading = conf.message_loading
group_at = conf.group_at


class Step:
    @staticmethod
    async def manual_step(manual_input: str, matcher: Matcher):
        try:
            if manual_input == "取消":
                await matcher.finish(Message("已取消手动刷步。"), at_sender=group_at)
                return
            if '，' in manual_input:
                await matcher.reject(Message(message_block_chinesecomma), at_sender=group_at)
                return
            user, password, steps = manual_input.split(',')
            if not steps.isdigit() or int(steps) > 98800:
                await matcher.reject(Message(message_block_step), at_sender=group_at)
                return
            await matcher.send(Message(message_loading), at_sender=group_at)
            params = {
                'user': user,
                'password': password,
                'steps': steps
            }
            logger.info(f"{params}")
            async with AsyncClient() as client:
                response = await client.get(url, params=params)
                response.raise_for_status()  # 如果响应状态码不是200，会抛出HTTPError异常
                result = response.json()
            module = load_module(result)
            message = message_success
            if handle_module:
                message += f"\n详情: {module}"
            await matcher.finish(Message(message), at_sender=group_at)
        except ValueError:
            await matcher.reject(Message("输入格式错误，请按照账号,密码,步数的格式输入。"))
        except httpx.RequestError as e:
            message = message_block_requesterror
            if handle_module:
                message += f"\n详情: {e}"
            await matcher.finish(Message(message), at_sender=group_at)
        except Exception as e:
            message = message_block_unknownerror
            if handle_module:
                message += f"\n详情: {e}"
            await matcher.finish(Message(message), at_sender=group_at)

    @staticmethod
    async def auto_step(steps: str, matcher: Matcher):
        if steps == "取消":
            await matcher.finish(Message("已取消自动刷步。"), at_sender=group_at)
            return
        elif not steps.isdigit() or int(steps) > 98800:
            await matcher.reject(Message(message_block_step), at_sender=group_at)
            return
        await matcher.send(Message(message_loading), at_sender=group_at)
        params = {
            'user': user,
            'password': password,
            'steps': steps
        }
        logger.info(f"{params}")
        try:
            async with AsyncClient() as client:
                response = await client.get(url, params=params)
                response.raise_for_status()  # 如果响应状态码不是200，会抛出HTTPError异常
                result = response.json()
            module = load_module(result)
            message = message_success
            if handle_module:
                message += f"\n详情: {module}"
            await matcher.finish(Message(message), at_sender=group_at)
        except httpx.RequestError as e:
            message = message_block_requesterror
            if handle_module:
                message += f"\n详情: {e}"
            await matcher.finish(Message(message), at_sender=group_at)
        except ValueError:
            message = "服务器返回了无效的数据，请稍后再试。"
            if handle_module:
                message += f"\n详情: {result}"
            await matcher.finish(Message(message), at_sender=group_at)



from nonebot import on_message
from nonebot.adapters.onebot.v11 import Message, MessageEvent, PrivateMessageEvent, GroupMessageEvent
from nonebot.internal.params import ArgPlainText
from nonebot.internal.rule import Rule
from nonebot.plugin import PluginMetadata
from nonebot.rule import to_me, command, is_type
from .ResultModule import load_check
from .Config import Config, conf
from .Step import Step

# ---------------------------Configurations---------------------------
__plugin_meta__ = PluginMetadata(
    name="nonebot-plugin-zepplife",
    description="基于调用xwteam平台专属api运行的机器人插件，目前仅支持Zepp、微信、支付宝刷步，后续还会更新其他功能",
    usage="",
    type='application',
    homepage="https://github.com/1296lol/nonebot-plugin-zepplife",
    config=Config,
    supported_adapters={"~onebot.v11"},
    extra={
        "author": "1296",
        "email": "hh1296@foxmail.com"
    }
)

user = conf.zepplife_user
password = conf.zepplife_password
private_chat = conf.private_chat
group_chat = conf.group_chat
group_at = conf.group_at
message_block_group = conf.message_block_group
message_block_private = conf.message_block_private
message_block_config = conf.message_block_config
message_block_users = conf.message_block_users
message_help = conf.message_help
handle_module = conf.handle_module
superusers = conf.superusers
only_superusers_used = conf.only_superusers_used
zepplife_commands = conf.zepplife_commands


# 命令转换函数
def trans_command(cmds: str | list[str]):
    if isinstance(cmds, str):
        cmds = [cmds]
    return command(*cmds)


# # 私聊阻断函数
# async def privateblock() -> bool:
#     return not private_chat
# block_private = is_type(PrivateMessageEvent) & privateblock
#
#
# private_block_matcher = on_message(rule=block_private, priority=50, block=True)
#
#
# @private_block_matcher.handle()
# async def private_chat_block():
#     await private_block_matcher.finish(Message(message_block_private))
#
#
# # 群聊阻断函数
# async def groupblock() -> bool:
#     return not group_chat
# block_group = is_type(GroupMessageEvent) & groupblock
# group_block_matcher = on_message(rule=block_group, priority=50, block=True)
#
#
# @group_block_matcher.handle()
# async def group_chat_block():
#     await group_block_matcher.finish(Message(message_block_group), at_sender=group_at)


# # 通用阻断函数
# async def block_message(event: MessageEvent) -> bool:
#     if isinstance(event, PrivateMessageEvent):
#         return not private_chat
#     elif isinstance(event, GroupMessageEvent):
#         return not group_chat
#     return False
#
#
# def command_block_rule(commands: dict[str, str | list[str]]) -> Rule:
#     async def rule(event: MessageEvent) -> bool:
#         if isinstance(event, MessageEvent):
#             for command, aliases in commands.items():
#                 if isinstance(aliases, str):
#                     aliases = [aliases]
#                 for alias in aliases:
#                     if event.get_plaintext().strip().startswith(f"/{alias}"):
#                         return await block_message(event)
#         return False
#     return Rule(rule)
#
#
# block_matcher = on_message(rule=command_block_rule(zepplife_commands), priority=50, block=True)
#
#
# @block_matcher.handle()
# async def handle_blocked_message(event: MessageEvent):
#     if isinstance(event, PrivateMessageEvent):
#         await block_matcher.finish(Message(message_block_private))
#     elif isinstance(event, GroupMessageEvent):
#         await block_matcher.finish(Message(message_block_group), at_sender=group_at)


# 刷步帮助
zepplife_help = to_me() & trans_command(zepplife_commands["help"])

matcher_help = on_message(rule=zepplife_help, priority=50, block=True)


@matcher_help.handle()
async def start():
    await matcher_help.finish(Message(message_help), at_sender=group_at)


# 刷步检查
zepplife_check = to_me() & trans_command(zepplife_commands["check"])

check = on_message(rule=zepplife_check, priority=50, block=True)


@check.handle()
async def checking():
    await check.finish(Message(f"检查结果:\n{load_check()}"), at_sender=group_at)


# 刷步实现
manual_step = to_me() & trans_command(zepplife_commands["manual_step"])
auto_step = to_me() & trans_command(zepplife_commands["auto_step"])

manual = on_message(rule=manual_step, priority=50, block=True)

auto = on_message(rule=auto_step, priority=50, block=True)


@manual.got("manual_input",
            prompt="请输入账号、密码、步数，格式为：账号,密码,步数。\n\n例如：abc@example.com,password,1000\n\n输入”取消“退出。")
async def handle_choice(event: MessageEvent, manual_input: str = ArgPlainText()):
    user_id = event.get_user_id()

    if not user or not password:
        # raise ValueError(message_block_config)
        await manual.finish(Message(message_block_config), at_sender=group_at)
        return

    if user_id not in superusers and only_superusers_used:
        await manual.finish(Message(message_block_users), at_sender=group_at)
        return

    if not private_chat:
        await manual.finish(Message(message_block_private), at_sender=group_at)
        return

    if not group_chat:
        await manual.finish(Message(message_block_group), at_sender=group_at)
        return

    await Step.manual_step(event, manual_input, manual)


@auto.got("steps", prompt="请输入步数，输入“取消”退出。")
async def handle_auto_step(event: MessageEvent, steps: str = ArgPlainText()):
    user_id = event.get_user_id()

    if not user or not password:
        # raise ValueError(message_block_config)
        await auto.finish(Message(message_block_config), at_sender=group_at)
        return

    if user_id not in superusers and only_superusers_used:
        await auto.finish(Message(message_block_users), at_sender=group_at)
        return

    if not private_chat:
        await auto.finish(Message(message_block_private), at_sender=group_at)
        return

    if not group_chat:
        await auto.finish(Message(message_block_group), at_sender=group_at)
        return

    await Step.auto_step(event, steps, auto)



from .Config import conf


# 详情模板
def load_module(data):
    # 可自行修改详情，美观建议
    module = (
        f"\n状态码: {data['code']}\n"
        f"状态信息: {data['msg']}\n"
        f"用户账号: {data['data']['user']}\n"
        f"密码: {data['data']['password']}\n"
        f"提示信息: {data['data']['steps']}\n"
        f"执行耗时: {data['exec_time']}秒\n"
        f"客户端IP: {data['ip']}\n"
        # f"接口作者: {data['debug']['author']}\n"
        # f"博客地址: {data['debug']['blog']}\n"
        # f"接口介绍: {data['debug']['server_info']}\n"
        # f"接口地址: {data['debug']['api_platform']}\n"
        # f"服务端通知: {data['debug']['notice']}\n"
        # f"服务端赞助: {data['debug']['sponsor']}\n"
        # f"服务端广告: {data['debug']['AD']}\n"
    )
    return module


# 检查模板
def load_check():
    module = (
        f"\n是否配置自动刷步账号:{bool(conf.zepplife_user)}\n"
        f"是否配置自动刷步密码:{bool(conf.zepplife_password)}\n"
        f"是否仅允许超级用户使用:{conf.only_superusers_used}\n"
        f"是否允许私聊:{conf.private_chat}\n"
        f"是否允许群聊:{conf.group_chat}\n"
        f"群聊回复是否艾特:{conf.group_at}\n"
        f"是否输出详情:{conf.handle_module}\n"
    )
    return module
